/*
 * pngrid - png2grid and grid2png converter
 * Copyright (C) 2009 Zhi Yan <yz@ai.univ-paris8.fr>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <cstdio>
#include <cstdlib>
#include "pngrid.hpp"

void png2grid(Png &png, Grid &grid) {
  printf("Png file format required: color_type = 0, bit_depth = 1\n");
  if((png.color_type != PNG_COLOR_TYPE_GRAY) || (png.bit_depth != 1)) {
    printf("I can't handle this file: color_type = %d, bit_depth = %d\n", png.info_ptr->color_type, png.bit_depth);
    exit(1);
  }
  printf("---> format test OK!\n");
  printf("---> height = %d, width = %d\n", png.height, png.width);
  
  int i, j, k;
  png_byte *row;
  
  for(i = 0; i < png.height; i++) {
    row = png.row_pointers[i];
    for(j = 0; j < png.width/8+1; j++) {
      for(k = 0; k < 8 && j*8+k < png.width; k++) {
	if(row[(j*8+k)/8] & (1 << (7 - (j*8+k) % 8))) {
	  grid.getData()[i][j*8+k] = FREE;
	  //printf(" ");
	} else {
	  grid.getData()[i][j*8+k] = OBST;
	  //printf("*");
	}
      }
    }
    //printf("\n");
  }
}

void grid2png(Png &png, Grid &grid, char *filename) {
  int i, j;
  png_byte *row;
  
  for(i = 0; i < grid.getHeight(); i++) {
    for(j = 0; j < grid.getWidth(); j++) {
      if(grid.getData()[i][j] == OBST) {
	row = png.row_pointers[i];
	row[j/8] &= ~(1 << (7-j%8));
      }
    }
  }
  
  // create output image file
  writePngFile(png, filename);
  printf("---> output image file saved by [%s]\n", filename);
}

void drawPixel(Grid &grid, int x, int y, int zone) {
  int i, j;
  
  for(j = y-zone; j <= y+zone; j++)
    for(i = x-zone; i <= x+zone; i++)
      grid.getData()[j][i] = OBST;
}
